@@ -25,6 +25,7 @@
 
 #include "openslide-hash.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <glib.h>
 
@@ -41,89 +42,53 @@ struct _openslide_hash *_openslide_hash_quickhash1_create(void) {
   return hash;
 }
 
+void _openslide_hash_data(struct _openslide_hash *hash, const void *data,
+                          int32_t datalen) {
+  if (hash && hash->enabled && data && datalen) {
+    g_checksum_update(hash->checksum, data, datalen);
+  }
+}
+
 void _openslide_hash_string(struct _openslide_hash *hash, const char *str) {
-  if (hash == NULL || !hash->enabled) {
-    return;
-  }
-
-  GChecksum *checksum = hash->checksum;
-
   const char *str_to_hash = str ? str : "";
-  g_checksum_update(checksum,
-		    (const guchar *) str_to_hash,
-		    strlen(str_to_hash) + 1);
+  _openslide_hash_data(hash, str_to_hash, strlen(str_to_hash) + 1);
 }
 
-bool _openslide_hash_tiff_tiles(struct _openslide_hash *hash, TIFF *tiff,
-                                GError **err) {
-  g_assert(TIFFIsTiled(tiff));
-
-  // get tile count
-  ttile_t count = TIFFNumberOfTiles(tiff);
-
-  // get tile sizes
-  toff_t *sizes;
-  if (TIFFGetField(tiff, TIFFTAG_TILEBYTECOUNTS, &sizes) == 0) {
-    g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_BAD_DATA,
-                "Cannot get tile size");
-    return false;  // ok, haven't allocated anything yet
-  }
-  toff_t total = 0;
-  for (ttile_t tile_no = 0; tile_no < count; tile_no++) {
-    total += sizes[tile_no];
-    if (total > (5 << 20)) {
-      // This is a non-pyramidal image or one with a very large top level.
-      // Refuse to calculate a quickhash for it to keep openslide_open()
-      // from taking an arbitrary amount of time.  (#79)
-      hash->enabled = false;
-      return true;  // ok, haven't allocated anything yet
-    }
-  }
-
-  // get offsets
-  toff_t *offsets;
-  if (TIFFGetField(tiff, TIFFTAG_TILEOFFSETS, &offsets) == 0) {
-    g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_BAD_DATA,
-                "Cannot get offsets");
-    return false;  // ok, haven't allocated anything yet
-  }
-
-  // hash each tile's raw data
-  const char *filename = TIFFFileName(tiff);
-  for (ttile_t tile_no = 0; tile_no < count; tile_no++) {
-    if (!_openslide_hash_file_part(hash, filename, offsets[tile_no], sizes[tile_no], err)) {
-      return false;
-    }
-  }
-
-  return true;
-}
-
-
 bool _openslide_hash_file(struct _openslide_hash *hash, const char *filename,
                           GError **err) {
-  int64_t size = _openslide_fsize(filename, err);
-  if (size == -1) {
-    return false;
-  }
-  return _openslide_hash_file_part(hash, filename, 0, size, err);
+  return _openslide_hash_file_part(hash, filename, 0, -1, err);
 }
 
 bool _openslide_hash_file_part(struct _openslide_hash *hash,
 			       const char *filename,
 			       int64_t offset, int64_t size,
 			       GError **err) {
+  bool success = false;
+
   FILE *f = _openslide_fopen(filename, "rb", err);
   if (f == NULL) {
     return false;
   }
 
+  if (size == -1) {
+    // hash to end of file
+    if (fseeko(f, 0, SEEK_END)) {
+      _openslide_io_error(err, "Couldn't seek %s", filename);
+      goto DONE;
+    }
+    int64_t len = ftello(f);
+    if (len == -1) {
+      _openslide_io_error(err, "Couldn't get size of %s", filename);
+      goto DONE;
+    }
+    size = len - offset;
+  }
+
   uint8_t buf[4096];
 
   if (fseeko(f, offset, SEEK_SET) == -1) {
     _openslide_io_error(err, "Can't seek in %s", filename);
-    fclose(f);
-    return false;
+    goto DONE;
   }
 
   int64_t bytes_left = size;
@@ -132,24 +97,30 @@ bool _openslide_hash_file_part(struct _openslide_hash *hash,
     int64_t bytes_read = fread(buf, 1, bytes_to_read, f);
 
     if (bytes_read != bytes_to_read) {
-      g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_BAD_DATA,
+      g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_FAILED,
                   "Can't read from %s", filename);
-      fclose(f);
-      return false;
+      goto DONE;
     }
 
     //    g_debug("hash '%s' %" G_GINT64_FORMAT " %d", filename, offset + (size - bytes_left), bytes_to_read);
 
     bytes_left -= bytes_read;
 
-    if (hash != NULL) {
-      GChecksum *checksum = hash->checksum;
-      g_checksum_update(checksum, buf, bytes_read);
-    }
+    _openslide_hash_data(hash, buf, bytes_read);
   }
 
+  success = true;
+
+DONE:
   fclose(f);
-  return true;
+  return success;
+}
+
+// Invalidate this hash.  Use if this slide is unhashable for some reason.
+void _openslide_hash_disable(struct _openslide_hash *hash) {
+  if (hash) {
+    hash->enabled = false;
+  }
 }
 
 const char *_openslide_hash_get_string(struct _openslide_hash *hash) {
