@@ -22,34 +22,33 @@
 #include <config.h>
 
 #include "openslide-private.h"
+#include "openslide-decode-tifflike.h"
 
 #include <stdlib.h>
 #include <string.h>
 
 #include <glib.h>
+#include <glib-object.h>
 #include <libxml/parser.h>
 
-#include "openslide-cache.h"
-#include "openslide-tilehelper.h"
 #include "openslide-cairo.h"
+#include "openslide-error.h"
 
-const char _openslide_release_info[] = "OpenSlide " PACKAGE_VERSION ", copyright (C) 2007-2012 Carnegie Mellon University and others.\nLicensed under the GNU Lesser General Public License, version 2.1.";
+const char _openslide_release_info[] = "OpenSlide " SUFFIXED_VERSION ", copyright (C) 2007-2014 Carnegie Mellon University and others.\nLicensed under the GNU Lesser General Public License, version 2.1.";
 
 static const char * const EMPTY_STRING_ARRAY[] = { NULL };
 
-static const _openslide_vendor_fn non_tiff_formats[] = {
-  _openslide_try_mirax,
-  _openslide_try_hamamatsu,
-  //_openslide_try_hamamatsu_ndpi, // it is a tiff format? but not to libtiff
-  NULL
-};
-
-static const _openslide_tiff_vendor_fn tiff_formats[] = {
-  _openslide_try_trestle,
-  _openslide_try_aperio,
-  _openslide_try_leica,
-  _openslide_try_generic_tiff,
-  NULL
+static const struct _openslide_format *formats[] = {
+  &_openslide_format_mirax,
+  &_openslide_format_hamamatsu_vms_vmu,
+  &_openslide_format_hamamatsu_ndpi,
+  &_openslide_format_sakura,
+  &_openslide_format_trestle,
+  &_openslide_format_aperio,
+  &_openslide_format_leica,
+  &_openslide_format_ventana,
+  &_openslide_format_generic_tiff,
+  NULL,
 };
 
 static bool openslide_was_dynamically_loaded;
@@ -57,18 +56,22 @@ static bool openslide_was_dynamically_loaded;
 // called from shared-library constructor!
 static void __attribute__((constructor)) _openslide_init(void) {
   // activate threads
-  if (!g_thread_supported ()) g_thread_init (NULL);
+  if (!g_thread_supported()) {
+    g_thread_init(NULL);
+  }
+  // initialize GObject
+  g_type_init();
+  // init libxml2
   xmlInitParser();
+  // parse debug options
+  _openslide_debug_init();
   openslide_was_dynamically_loaded = true;
 }
 
 static void destroy_associated_image(gpointer data) {
   struct _openslide_associated_image *img = data;
 
-  if (img->destroy_ctx != NULL && img->ctx != NULL) {
-    img->destroy_ctx(img->ctx);
-  }
-  g_slice_free(struct _openslide_associated_image, img);
+  img->ops->destroy(img);
 }
 
 static bool level_in_range(openslide_t *osr, int32_t level) {
@@ -83,135 +86,117 @@ static bool level_in_range(openslide_t *osr, int32_t level) {
   return true;
 }
 
-static void reset_osr(openslide_t *osr) {
-  if (osr) {
-    g_hash_table_remove_all(osr->properties);
-    g_hash_table_remove_all(osr->associated_images);
-  }
+static openslide_t *create_osr(void) {
+  openslide_t *osr = g_slice_new0(openslide_t);
+  osr->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                          g_free, g_free);
+  osr->associated_images = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                 g_free,
+                                                 destroy_associated_image);
+  return osr;
 }
 
-static void init_quickhash1_out(struct _openslide_hash **quickhash1_OUT) {
+static const struct _openslide_format *detect_format(const char *filename,
+                                                     struct _openslide_tifflike **tl_OUT) {
+  GError *tmp_err = NULL;
+
+  struct _openslide_tifflike *tl = _openslide_tifflike_create(filename,
+                                                              &tmp_err);
+  if (!tl) {
+    if (_openslide_debug(OPENSLIDE_DEBUG_DETECTION)) {
+      g_message("tifflike: %s", tmp_err->message);
+    }
+    g_clear_error(&tmp_err);
+  }
+
+  for (const struct _openslide_format **cur = formats; *cur; cur++) {
+    const struct _openslide_format *format = *cur;
+
+    g_assert(format->name && format->vendor &&
+             format->detect && format->open);
+
+    if (format->detect(filename, tl, &tmp_err)) {
+      // success!
+      if (tl_OUT) {
+        *tl_OUT = tl;
+      } else {
+        _openslide_tifflike_destroy(tl);
+      }
+      return format;
+    }
+
+    // reset for next format
+    if (_openslide_debug(OPENSLIDE_DEBUG_DETECTION)) {
+      g_message("%s: %s", format->name, tmp_err->message);
+    }
+    g_clear_error(&tmp_err);
+  }
+
+  // no match
+  _openslide_tifflike_destroy(tl);
+  return NULL;
+}
+
+static bool open_backend(openslide_t *osr,
+                         const struct _openslide_format *format,
+                         const char *filename,
+                         struct _openslide_tifflike *tl,
+                         struct _openslide_hash **quickhash1_OUT,
+                         GError **err) {
   if (quickhash1_OUT) {
     *quickhash1_OUT = _openslide_hash_quickhash1_create();
   }
-}
 
-static void free_quickhash1_if_failed(bool result,
-				      struct _openslide_hash **quickhash1_OUT) {
+  bool result = format->open(osr, filename, tl,
+                             quickhash1_OUT ? *quickhash1_OUT : NULL,
+                             err);
+
+  // check for error-handling bugs in open function
+  if (!result && err && !*err) {
+    g_warning("%s opener failed without setting error", format->name);
+    // assume the worst
+    g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_FAILED,
+                "Unknown error");
+  }
+  if (result && err && *err) {
+    g_warning("%s opener succeeded but set error", format->name);
+    result = false;
+  }
+
   // if we have a hash and a false result, destroy
   if (quickhash1_OUT && !result) {
     _openslide_hash_destroy(*quickhash1_OUT);
   }
-}
-
-static void fixup_format_error(bool result, GError **err) {
-  // check for error-handling bugs in detector
-
-  if (!result && err && !*err) {
-    g_warning("Format detector failed without setting error");
-    // assume the worst
-    g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_BAD_DATA,
-                "Unknown error");
-  }
-
-  if (result && err && *err) {
-    g_warning("Format detector succeeded but set error: %s", (*err)->message);
-    g_clear_error(err);
-  }
-}
-
-static bool try_format(openslide_t *osr, const char *filename,
-		       struct _openslide_hash **quickhash1_OUT,
-		       const _openslide_vendor_fn *format_check,
-		       GError **err) {
-  reset_osr(osr);
-  init_quickhash1_out(quickhash1_OUT);
-
-  bool result = (*format_check)(osr, filename,
-                                quickhash1_OUT ? *quickhash1_OUT : NULL,
-                                err);
-
-  free_quickhash1_if_failed(result, quickhash1_OUT);
-  fixup_format_error(result, err);
 
   return result;
 }
 
-static bool try_tiff_format(openslide_t *osr, TIFF *tiff,
-			    struct _openslide_hash **quickhash1_OUT,
-			    const _openslide_tiff_vendor_fn *format_check,
-			    GError **err) {
-  reset_osr(osr);
-  init_quickhash1_out(quickhash1_OUT);
+const char *openslide_detect_vendor(const char *filename) {
+  g_assert(openslide_was_dynamically_loaded);
 
-  TIFFSetDirectory(tiff, 0);
-  bool result = (*format_check)(osr, tiff,
-                                quickhash1_OUT ? *quickhash1_OUT : NULL,
-                                err);
-
-  free_quickhash1_if_failed(result, quickhash1_OUT);
-  fixup_format_error(result, err);
-
-  return result;
-}
-
-static bool try_all_formats(openslide_t *osr, const char *filename,
-			    struct _openslide_hash **quickhash1_OUT,
-			    GError **err) {
-  GError *tmp_err = NULL;
-
-  // non-tiff
-  const _openslide_vendor_fn *fn = non_tiff_formats;
-  while (*fn) {
-    if (try_format(osr, filename, quickhash1_OUT, fn, &tmp_err)) {
-      return true;
-    }
-    if (!g_error_matches(tmp_err, OPENSLIDE_ERROR,
-                         OPENSLIDE_ERROR_FORMAT_NOT_SUPPORTED)) {
-      g_propagate_error(err, tmp_err);
-      return false;
-    }
-    //g_debug("%s", tmp_err->message);
-    g_clear_error(&tmp_err);
-    fn++;
+  const struct _openslide_format *format = detect_format(filename, NULL);
+  if (!format) {
+    return NULL;
   }
-
-
-  // tiff
-  TIFF *tiff = _openslide_tiff_open(filename);
-  if (tiff != NULL) {
-    const _openslide_tiff_vendor_fn *tfn = tiff_formats;
-    while (*tfn) {
-      if (try_tiff_format(osr, tiff, quickhash1_OUT, tfn, &tmp_err)) {
-	return true;
-      }
-      if (!g_error_matches(tmp_err, OPENSLIDE_ERROR,
-                           OPENSLIDE_ERROR_FORMAT_NOT_SUPPORTED)) {
-        g_propagate_error(err, tmp_err);
-        TIFFClose(tiff);
-        return false;
-      }
-      //g_debug("%s", tmp_err->message);
-      g_clear_error(&tmp_err);
-      tfn++;
-    }
-
-    // close only if failed
-    TIFFClose(tiff);
-  }
-
-
-  // no match
-  g_set_error(err, OPENSLIDE_ERROR, OPENSLIDE_ERROR_FORMAT_NOT_SUPPORTED,
-              "Format not recognized");
-  return false;
+  return format->vendor;
 }
 
 bool openslide_can_open(const char *filename) {
   g_assert(openslide_was_dynamically_loaded);
 
-  // quick test
-  return try_all_formats(NULL, filename, NULL, NULL);
+  // detect format
+  struct _openslide_tifflike *tl;
+  const struct _openslide_format *format = detect_format(filename, &tl);
+  if (!format) {
+    return false;
+  }
+
+  // try opening
+  openslide_t *osr = create_osr();
+  bool success = open_backend(osr, format, filename, tl, NULL, NULL);
+  _openslide_tifflike_destroy(tl);
+  openslide_close(osr);
+  return success;
 }
 
 
@@ -249,59 +234,52 @@ openslide_t *openslide_open(const char *filename) {
 
   g_assert(openslide_was_dynamically_loaded);
 
-  // alloc memory
-  openslide_t *osr = g_slice_new0(openslide_t);
-  osr->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
-					  g_free, g_free);
-  osr->associated_images = g_hash_table_new_full(g_str_hash, g_str_equal,
-						 g_free, destroy_associated_image);
-
-  // try to read it
-  struct _openslide_hash *quickhash1 = NULL;
-  if (!try_all_formats(osr, filename, &quickhash1, &tmp_err)) {
-    // failure
-    if (g_error_matches(tmp_err, OPENSLIDE_ERROR,
-                        OPENSLIDE_ERROR_FORMAT_NOT_SUPPORTED)) {
-      // not a slide file
-      g_clear_error(&tmp_err);
-      openslide_close(osr);
-      return NULL;
-    } else {
-      // failed to read slide
-      _openslide_set_error(osr, "%s", tmp_err->message);
-      g_clear_error(&tmp_err);
-      return osr;
-    }
+  // detect format
+  struct _openslide_tifflike *tl;
+  const struct _openslide_format *format = detect_format(filename, &tl);
+  if (!format) {
+    // not a slide file
+    return NULL;
   }
 
+  // alloc memory
+  openslide_t *osr = create_osr();
+
+  // open backend
+  struct _openslide_hash *quickhash1 = NULL;
+  bool success = open_backend(osr, format, filename, tl, &quickhash1,
+                              &tmp_err);
+  _openslide_tifflike_destroy(tl);
+  if (!success) {
+    // failed to read slide
+    _openslide_propagate_error(osr, tmp_err);
+    return osr;
+  }
+  g_assert(osr->levels);
+
   // compute downsamples if not done already
   int64_t blw, blh;
   openslide_get_level0_dimensions(osr, &blw, &blh);
 
-  if (!osr->downsamples) {
-    osr->downsamples = g_new0(double, osr->level_count);
-  }
-  if (osr->downsamples[0] == 0) {
-    osr->downsamples[0] = 1.0;
+  if (osr->level_count && osr->levels[0]->downsample == 0) {
+    osr->levels[0]->downsample = 1.0;
   }
   for (int32_t i = 1; i < osr->level_count; i++) {
-    if (osr->downsamples[i] == 0) {
-      int64_t w, h;
-      openslide_get_level_dimensions(osr, i, &w, &h);
-
-      osr->downsamples[i] =
-        (((double) blh / (double) h) +
-         ((double) blw / (double) w)) / 2.0;
+    struct _openslide_level *l = osr->levels[i];
+    if (l->downsample == 0) {
+      l->downsample =
+        (((double) blh / (double) l->h) +
+         ((double) blw / (double) l->w)) / 2.0;
     }
   }
 
   // check downsamples
   for (int32_t i = 1; i < osr->level_count; i++) {
-    //g_debug("downsample: %g", osr->downsamples[i]);
+    //g_debug("downsample: %g", osr->levels[i]->downsample);
 
-    if (osr->downsamples[i] < osr->downsamples[i - 1]) {
+    if (osr->levels[i]->downsample < osr->levels[i - 1]->downsample) {
       g_warning("Downsampled images not correctly ordered: %g < %g",
-		osr->downsamples[i], osr->downsamples[i - 1]);
+		osr->levels[i]->downsample, osr->levels[i - 1]->downsample);
       openslide_close(osr);
       _openslide_hash_destroy(quickhash1);
       return NULL;
@@ -318,41 +296,41 @@ openslide_t *openslide_open(const char *filename) {
   _openslide_hash_destroy(quickhash1);
 
   // set other properties
+  g_hash_table_insert(osr->properties,
+                      g_strdup(OPENSLIDE_PROPERTY_NAME_VENDOR),
+                      g_strdup(format->vendor));
   g_hash_table_insert(osr->properties,
 		      g_strdup(_OPENSLIDE_PROPERTY_NAME_LEVEL_COUNT),
 		      g_strdup_printf("%d", osr->level_count));
   bool should_have_geometry;
   for (int32_t i = 0; i < osr->level_count; i++) {
-    int64_t w, h;
-    openslide_get_level_dimensions(osr, i, &w, &h);
+    struct _openslide_level *l = osr->levels[i];
 
     g_hash_table_insert(osr->properties,
 			g_strdup_printf(_OPENSLIDE_PROPERTY_NAME_TEMPLATE_LEVEL_WIDTH, i),
-			g_strdup_printf("%" G_GINT64_FORMAT, w));
+			g_strdup_printf("%" G_GINT64_FORMAT, l->w));
     g_hash_table_insert(osr->properties,
 			g_strdup_printf(_OPENSLIDE_PROPERTY_NAME_TEMPLATE_LEVEL_HEIGHT, i),
-			g_strdup_printf("%" G_GINT64_FORMAT, h));
+			g_strdup_printf("%" G_GINT64_FORMAT, l->h));
     g_hash_table_insert(osr->properties,
 			g_strdup_printf(_OPENSLIDE_PROPERTY_NAME_TEMPLATE_LEVEL_DOWNSAMPLE, i),
-			_openslide_format_double(osr->downsamples[i]));
+			_openslide_format_double(l->downsample));
 
     // tile geometry
-    w = h = -1;
-    (osr->ops->get_tile_geometry)(osr, i, &w, &h);
-    bool have_geometry = (w > 0 && h > 0);
+    bool have_geometry = (l->tile_w > 0 && l->tile_h > 0);
     if (i == 0) {
       should_have_geometry = have_geometry;
     }
-    if (!openslide_get_error(osr) && have_geometry != should_have_geometry) {
+    if (have_geometry != should_have_geometry) {
       g_warning("Inconsistent tile geometry hints between levels");
     }
     if (have_geometry) {
       g_hash_table_insert(osr->properties,
                           g_strdup_printf(_OPENSLIDE_PROPERTY_NAME_TEMPLATE_LEVEL_TILE_WIDTH, i),
-                          g_strdup_printf("%" G_GINT64_FORMAT, w));
+                          g_strdup_printf("%" G_GINT64_FORMAT, l->tile_w));
       g_hash_table_insert(osr->properties,
                           g_strdup_printf(_OPENSLIDE_PROPERTY_NAME_TEMPLATE_LEVEL_TILE_HEIGHT, i),
-                          g_strdup_printf("%" G_GINT64_FORMAT, h));
+                          g_strdup_printf("%" G_GINT64_FORMAT, l->tile_h));
     }
   }
 
@@ -364,9 +342,6 @@ openslide_t *openslide_open(const char *filename) {
   osr->cache = _openslide_cache_create(_OPENSLIDE_USEFUL_CACHE_SIZE);
   //osr->cache = _openslide_cache_create(0);
 
-  // validate required properties
-  g_assert(openslide_get_property_value(osr, OPENSLIDE_PROPERTY_NAME_VENDOR));
-
   return osr;
 }
 
@@ -382,8 +357,6 @@ void openslide_close(openslide_t *osr) {
   g_free(osr->associated_image_names);
   g_free(osr->property_names);
 
-  g_free(osr->downsamples);
-
   if (osr->cache) {
     _openslide_cache_destroy(osr->cache);
   }
@@ -412,7 +385,8 @@ void openslide_get_level_dimensions(openslide_t *osr, int32_t level,
     return;
   }
 
-  (osr->ops->get_dimensions)(osr, level, w, h);
+  *w = osr->levels[level]->w;
+  *h = osr->levels[level]->h;
 }
 
 void openslide_get_layer0_dimensions(openslide_t *osr,
@@ -451,13 +425,13 @@ int32_t openslide_get_best_level_for_downsample(openslide_t *osr,
   }
 
   // too small, return first
-  if (downsample < osr->downsamples[0]) {
+  if (downsample < osr->levels[0]->downsample) {
     return 0;
   }
 
   // find where we are in the middle
   for (int32_t i = 1; i < osr->level_count; i++) {
-    if (downsample < osr->downsamples[i]) {
+    if (downsample < osr->levels[i]->downsample) {
       return i - 1;
     }
   }
@@ -477,7 +451,7 @@ double openslide_get_level_downsample(openslide_t *osr, int32_t level) {
     return -1.0;
   }
 
-  return osr->downsamples[level];
+  return osr->levels[level]->downsample;
 }
 
 double openslide_get_layer_downsample(openslide_t *osr, int32_t level) {
@@ -500,11 +474,14 @@ void openslide_cancel_prefetch_hint(openslide_t *osr G_GNUC_UNUSED,
   g_warning("openslide_cancel_prefetch_hint has never been implemented and should not be called");
 }
 
-static void read_region(openslide_t *osr,
+static bool read_region(openslide_t *osr,
 			cairo_t *cr,
 			int64_t x, int64_t y,
 			int32_t level,
-			int64_t w, int64_t h) {
+			int64_t w, int64_t h,
+			GError **err) {
+  bool success = true;
+
   // save the old pattern, it's the only thing push/pop won't restore
   cairo_pattern_t *old_source = cairo_get_source(cr);
   cairo_pattern_reference(old_source);
@@ -521,8 +498,10 @@ static void read_region(openslide_t *osr,
   cairo_set_operator(cr, CAIRO_OPERATOR_SATURATE);
 
   if (level_in_range(osr, level)) {
+    struct _openslide_level *l = osr->levels[level];
+
     // offset if given negative coordinates
-    double ds = openslide_get_level_downsample(osr, level);
+    double ds = l->downsample;
     int64_t tx = 0;
     int64_t ty = 0;
     if (x < 0) {
@@ -539,13 +518,13 @@ static void read_region(openslide_t *osr,
 
     // paint
     if (w > 0 && h > 0) {
-      (osr->ops->paint_region)(osr, cr, x, y, level, w, h);
+      success = osr->ops->paint_region(osr, cr, x, y, l, w, h, err);
     }
   }
 
   cairo_pop_group_to_source(cr);
 
-  if (!openslide_get_error(osr)) {
+  if (success) {
     // commit, nothing went wrong
     cairo_paint(cr);
   }
@@ -553,13 +532,17 @@ static void read_region(openslide_t *osr,
   // restore old source
   cairo_set_source(cr, old_source);
   cairo_pattern_destroy(old_source);
+
+  return success;
 }
 
 static bool ensure_nonnegative_dimensions(openslide_t *osr, int64_t w, int64_t h) {
   if (w < 0 || h < 0) {
-    _openslide_set_error(osr,
-			 "negative width (%" G_GINT64_FORMAT ") or negative height (%"
-			 G_GINT64_FORMAT ") not allowed", w, h);
+    GError *tmp_err = g_error_new(OPENSLIDE_ERROR, OPENSLIDE_ERROR_FAILED,
+                                  "negative width "
+                                  "(%"G_GINT64_FORMAT") or negative height "
+                                  "(%"G_GINT64_FORMAT") not allowed", w, h);
+    _openslide_propagate_error(osr, tmp_err);
     return false;
   }
   return true;
@@ -570,6 +553,8 @@ void openslide_read_region(openslide_t *osr,
 			   int64_t x, int64_t y,
 			   int32_t level,
 			   int64_t w, int64_t h) {
+  GError *tmp_err = NULL;
+
   if (!ensure_nonnegative_dimensions(osr, w, h)) {
     return;
   }
@@ -593,10 +578,8 @@ void openslide_read_region(openslide_t *osr,
   //    amount of RAM.
   const int64_t d = 4096;
   double ds = openslide_get_level_downsample(osr, level);
-  for (int64_t row = 0; !openslide_get_error(osr) && row < (h + d - 1) / d;
-          row++) {
-    for (int64_t col = 0; !openslide_get_error(osr) && col < (w + d - 1) / d;
-            col++) {
+  for (int64_t row = 0; row < (h + d - 1) / d; row++) {
+    for (int64_t col = 0; col < (w + d - 1) / d; col++) {
       // calculate surface coordinates and size
       int64_t sx = x + col * d * ds;     // level 0 plane
       int64_t sy = y + row * d * ds;     // level 0 plane
@@ -619,17 +602,28 @@ void openslide_read_region(openslide_t *osr,
       cairo_surface_destroy(surface);
 
       // paint
-      read_region(osr, cr, sx, sy, level, sw, sh);
+      if (!read_region(osr, cr, sx, sy, level, sw, sh, &tmp_err)) {
+        cairo_destroy(cr);
+        goto OUT;
+      }
 
       // done
-      _openslide_check_cairo_status_possibly_set_error(osr, cr);
+      if (!_openslide_check_cairo_status(cr, &tmp_err)) {
+        cairo_destroy(cr);
+        goto OUT;
+      }
+
       cairo_destroy(cr);
     }
   }
 
-  // ensure we don't return a partial result
-  if (openslide_get_error(osr)) {
-    memset(dest, 0, w * h * 4);
+OUT:
+  if (tmp_err) {
+    _openslide_propagate_error(osr, tmp_err);
+    if (dest) {
+      // ensure we don't return a partial result
+      memset(dest, 0, w * h * 4);
+    }
   }
 }
 
@@ -639,6 +633,8 @@ void openslide_cairo_read_region(openslide_t *osr,
 				 int64_t x, int64_t y,
 				 int32_t level,
 				 int64_t w, int64_t h) {
+  GError *tmp_err = NULL;
+
   if (!ensure_nonnegative_dimensions(osr, w, h)) {
     return;
   }
@@ -647,9 +643,13 @@ void openslide_cairo_read_region(openslide_t *osr,
     return;
   }
 
-  read_region(osr, cr, x, y, level, w, h);
+  if (read_region(osr, cr, x, y, level, w, h, &tmp_err)) {
+    _openslide_check_cairo_status(cr, &tmp_err);
+  }
 
-  _openslide_check_cairo_status_possibly_set_error(osr, cr);
+  if (tmp_err) {
+    _openslide_propagate_error(osr, tmp_err);
+  }
 }
 
 
@@ -697,6 +697,8 @@ void openslide_get_associated_image_dimensions(openslide_t *osr, const char *nam
 void openslide_read_associated_image(openslide_t *osr,
 				     const char *name,
 				     uint32_t *dest) {
+  GError *tmp_err = NULL;
+
   if (openslide_get_error(osr)) {
     return;
   }
@@ -709,9 +711,12 @@ void openslide_read_associated_image(openslide_t *osr,
     size_t pixels = img->w * img->h;
     uint32_t *buf = g_new(uint32_t, pixels);
 
-    img->get_argb_data(osr, img->ctx, buf, img->w, img->h);
-    if (dest && !openslide_get_error(osr)) {
-      memcpy(dest, buf, pixels * sizeof(uint32_t));
+    if (img->ops->get_argb_data(img, buf, &tmp_err)) {
+      if (dest) {
+        memcpy(dest, buf, pixels * sizeof(uint32_t));
+      }
+    } else {
+      _openslide_propagate_error(osr, tmp_err);
     }
 
     g_free(buf);
@@ -719,5 +724,5 @@ void openslide_read_associated_image(openslide_t *osr,
 }
 
 const char *openslide_get_version(void) {
-  return PACKAGE_VERSION;
+  return SUFFIXED_VERSION;
 }
