@@ -1,7 +1,7 @@
 /*
  *  OpenSlide, a library for reading whole slide image files
  *
- *  Copyright (c) 2007-2012 Carnegie Mellon University
+ *  Copyright (c) 2007-2014 Carnegie Mellon University
  *  All rights reserved.
  *
  *  OpenSlide is free software: you can redistribute it and/or modify
@@ -32,7 +32,6 @@
 
 #include <openslide-features.h>
 
-#include <stdio.h>
 #include <stdint.h>
 
 #ifdef __cplusplus
@@ -52,24 +51,33 @@ typedef struct _openslide openslide_t;
 //@{
 
 /**
- * Do a quick check to see if a whole slide image is valid.
+ * Quickly determine whether a whole slide image is recognized.
  *
- * This function is more efficient than openslide_open() if you do not
- * necessarily intend to open the specified file (for example, when creating
- * a list of available slide files).  If you definitely intend to open the
- * file, calling this function will only reduce performance; you should call
- * openslide_open() directly.
+ * If OpenSlide recognizes the file referenced by @p filename, return a
+ * string identifying the slide format vendor.  This is equivalent to the
+ * value of the #OPENSLIDE_PROPERTY_NAME_VENDOR property.  Calling
+ * openslide_open() on this file will return a valid OpenSlide object or
+ * an OpenSlide object in error state.
+ *
+ * Otherwise, return NULL.  Calling openslide_open() on this file will also
+ * return NULL.
  *
  * @param filename The filename to check.
- * @return If openslide_open() will succeed.
+ * @return An identification of the format vendor for this file, or NULL.
+ * @since 3.4.0
  */
 OPENSLIDE_PUBLIC()
-bool openslide_can_open(const char *filename);
+const char *openslide_detect_vendor(const char *filename);
 
 
 /**
  * Open a whole slide image.
  *
+ * This function can be expensive; avoid calling it unnecessarily.  For
+ * example, a tile server should not call openslide_open() on every tile
+ * request.  Instead, it should maintain a cache of OpenSlide objects and
+ * reuse them when possible.
+ *
  * @param filename The filename to open.
  * @return
  *         On success, a new OpenSlide object.
@@ -278,6 +286,38 @@ const char *openslide_get_error(openslide_t *osr);
  * @since 3.3.0
  */
 #define OPENSLIDE_PROPERTY_NAME_MPP_Y "openslide.mpp-y"
+
+/**
+ * The name of the property containing the X coordinate of the rectangle
+ * bounding the non-empty region of the slide, if available.
+ *
+ * @since 3.4.0
+ */
+#define OPENSLIDE_PROPERTY_NAME_BOUNDS_X "openslide.bounds-x"
+
+/**
+ * The name of the property containing the Y coordinate of the rectangle
+ * bounding the non-empty region of the slide, if available.
+ *
+ * @since 3.4.0
+ */
+#define OPENSLIDE_PROPERTY_NAME_BOUNDS_Y "openslide.bounds-y"
+
+/**
+ * The name of the property containing the width of the rectangle bounding
+ * the non-empty region of the slide, if available.
+ *
+ * @since 3.4.0
+ */
+#define OPENSLIDE_PROPERTY_NAME_BOUNDS_WIDTH "openslide.bounds-width"
+
+/**
+ * The name of the property containing the height of the rectangle bounding
+ * the non-empty region of the slide, if available.
+ *
+ * @since 3.4.0
+ */
+#define OPENSLIDE_PROPERTY_NAME_BOUNDS_HEIGHT "openslide.bounds-height"
 //@}
 
 /**
@@ -410,6 +450,30 @@ const char *openslide_get_version(void);
  */
 //@{
 
+/**
+ * Return whether openslide_open() will succeed.
+ *
+ * This function returns @p true if openslide_open() will return a valid
+ * @ref openslide_t, or @p false if it will return NULL or an
+ * @ref openslide_t in error state.  As such, there's no reason to use it;
+ * just call openslide_open().  For a less-expensive test that provides
+ * weaker guarantees, see openslide_detect_vendor().
+ *
+ * Before version 3.4.0, this function could be slightly faster than calling
+ * openslide_open(), but it could also erroneously return @p true in some
+ * cases where openslide_open() would fail.
+ *
+ * @param filename The filename to check.
+ * @return If openslide_open() will succeed.
+ * @deprecated Use openslide_detect_vendor() to efficiently check whether
+ *             a slide file is recognized by OpenSlide, or just call
+ *             openslide_open().
+ */
+OPENSLIDE_PUBLIC()
+OPENSLIDE_DEPRECATED_FOR(openslide_detect_vendor or openslide_open)
+bool openslide_can_open(const char *filename);
+
+
 /**
  * Get the number of levels in the whole slide image.
  *
