@@ -1,7 +1,7 @@
 /*
  *  OpenSlide, a library for reading whole slide image files
  *
- *  Copyright (c) 2007-2011 Carnegie Mellon University
+ *  Copyright (c) 2007-2014 Carnegie Mellon University
  *  All rights reserved.
  *
  *  OpenSlide is free software: you can redistribute it and/or modify
@@ -34,11 +34,6 @@
 #include <stdbool.h>
 #include <sys/time.h>
 
-#ifndef WIN32
-#include <sys/types.h>
-#include <unistd.h>
-#endif
-
 #include <glib.h>
 #include <cairo.h>
 #include <cairo-pdf.h>
@@ -60,6 +55,7 @@ static void test_next_biggest(openslide_t *osr, double downsample) {
 	 downsample, level, openslide_get_level_downsample(osr, level));
 }
 
+/*
 static void test_tile_walk(openslide_t *osr,
 			   int64_t tile_size) {
   printf("test_tile_walk: %" G_GINT64_FORMAT "\n", tile_size);
@@ -81,6 +77,7 @@ static void test_tile_walk(openslide_t *osr,
 
   free(buf);
 }
+*/
 
 static uint8_t apply_alpha(uint8_t s, uint8_t a, uint8_t d) {
   double ss = s / 255.0;
@@ -160,6 +157,7 @@ static void test_image_fetch(openslide_t *osr,
   }
 }
 
+/*
 static void test_horizontal_walk(openslide_t *osr,
 				 int64_t start_x,
 				 int64_t y,
@@ -202,7 +200,6 @@ static void test_vertical_walk(openslide_t *osr,
   free(buf);
 }
 
-/*
 static void test_pdf(openslide_t *osr, const char *filename) {
   printf("test_pdf: %s\n", filename);
   cairo_surface_t *pdf = cairo_pdf_surface_create(filename, 0, 0);
@@ -240,99 +237,18 @@ static void test_pdf(openslide_t *osr, const char *filename) {
 }
 */
 
-#ifndef WIN32
-static gint leak_test_running;  /* atomic ops only */
-
-static gpointer cloexec_thread(const gpointer prog) {
-  GHashTable *seen = g_hash_table_new_full(g_str_hash, g_str_equal,
-        g_free, NULL);
-  gchar *argv[] = {prog, "--leak-check--", NULL};
-
-  while (g_atomic_int_get(&leak_test_running)) {
-    gchar *out;
-    if (!g_spawn_sync(NULL, argv, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN |
-          G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL,
-          &out, NULL, NULL, NULL)) {
-      g_assert_not_reached();
-    }
-
-    gchar **lines = g_strsplit(out, "\n", 0);
-    for (gchar **line = lines; *line != NULL; line++) {
-      if (**line == 0) {
-        continue;
-      }
-      if (g_hash_table_lookup(seen, *line) == NULL) {
-        printf("Exec child received leaked fd to %s\n", *line);
-        g_hash_table_insert(seen, g_strdup(*line), (void *) 1);
-      }
-    }
-    g_strfreev(lines);
-    g_free(out);
-  }
-
-  g_hash_table_destroy(seen);
-  return NULL;
-}
-
-static void child_check_open_fds(void) {
-  for (int i = 3; i < 128; i++) {
-    gchar *proc = g_strdup_printf("/proc/%d/fd/%d", getpid(), i);
-    gchar *link = g_file_read_link(proc, NULL);
-    g_free(proc);
-    if (link != NULL) {
-      printf("%s\n", link);
-      g_free(link);
-    }
-  }
-}
-
-static void check_cloexec_leaks(const char *slide, void *prog)
-{
-  g_atomic_int_set(&leak_test_running, 1);
-  GThread *thr = g_thread_create(cloexec_thread, prog, TRUE, NULL);
-  g_assert(thr != NULL);
-  guint32 buf[512 * 512];
-  GTimer *timer = g_timer_new();
-  while (g_timer_elapsed(timer, NULL) < 2) {
-    openslide_t *osr = openslide_open(slide);
-    openslide_read_region(osr, buf, 0, 0, 0, 512, 512);
-    openslide_close(osr);
-  }
-  g_timer_destroy(timer);
-  g_atomic_int_set(&leak_test_running, 0);
-  g_thread_join(thr);
-}
-#else /* WIN32 */
-static void child_check_open_fds(void) {}
-
-static void check_cloexec_leaks(const char *slide, void *prog) {
-  (void) slide;
-  (void) prog;
-}
-#endif /* WIN32 */
-
-
 int main(int argc, char **argv) {
-  if (!g_thread_supported()) {
-    g_thread_init(NULL);
-  }
-
   if (argc != 2) {
     printf("give file!\n");
     return 1;
   }
 
-  if (g_str_equal(argv[1], "--leak-check--")) {
-    child_check_open_fds();
-    return 0;
-  }
-
   //  struct timeval start_tv;
   //  struct timeval end_tv;
 
   printf("version: %s\n", openslide_get_version());
 
-  printf("openslide_can_open returns %s\n", openslide_can_open(argv[1]) ? "true" : "false");
+  printf("openslide_detect_vendor returns %s\n", openslide_detect_vendor(argv[1]));
   openslide_t *osr = openslide_open(argv[1]);
 
   int64_t w, h;
@@ -468,6 +384,15 @@ int main(int argc, char **argv) {
   //test_image_fetch(osr, "test6", 0, h - 20, 100, 40, skip);
   test_image_fetch(osr, "test7", 0, 0, 200, 200, skip);
 
+  // active region
+  const char *bounds_x = openslide_get_property_value(osr, OPENSLIDE_PROPERTY_NAME_BOUNDS_X);
+  const char *bounds_y = openslide_get_property_value(osr, OPENSLIDE_PROPERTY_NAME_BOUNDS_Y);
+  if (bounds_x && bounds_y) {
+    int64_t x = g_ascii_strtoll(bounds_x, NULL, 10);
+    int64_t y = g_ascii_strtoll(bounds_y, NULL, 10);
+    test_image_fetch(osr, "test8", x, y, 200, 200, skip);
+  }
+
   //  test_pdf(osr, "test0.pdf");
 
 #ifdef HAVE_VALGRIND
@@ -476,7 +401,5 @@ int main(int argc, char **argv) {
 
   openslide_close(osr);
 
-  check_cloexec_leaks(argv[1], argv[0]);
-
   return 0;
 }
