@@ -29,9 +29,12 @@
 #include "openslide.h"
 
 #define MAX_FDS 128
+#define TIME_ITERATIONS 5
 
+static gchar *vendor_check;
 static gchar **prop_checks;
 static gchar **region_checks;
+static gboolean time_check;
 
 static gboolean have_error = FALSE;
 
@@ -115,28 +118,27 @@ static void check_regions(openslide_t *osr) {
 }
 
 static GOptionEntry options[] = {
+  {"vendor", 'n', 0, G_OPTION_ARG_STRING, &vendor_check,
+   "Check for specified vendor (\"none\" for NULL)", "\"VENDOR\""},
   {"property", 'p', 0, G_OPTION_ARG_STRING_ARRAY, &prop_checks,
    "Check for specified property value", "\"NAME=VALUE\""},
   {"region", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &region_checks,
    "Read specified region", "\"X Y LEVEL W H\""},
-  {NULL},
+  {"time", 't', 0, G_OPTION_ARG_NONE, &time_check,
+   "Report open time", NULL},
+  {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}
 };
 
 int main(int argc, char **argv) {
-  GOptionContext *ctx;
-  GError *err = NULL;
-  openslide_t *osr;
-  const char *filename;
-  GHashTable *fds;
-  struct stat st;
-  bool can_open;
+  GError *tmp_err = NULL;
 
   // Parse arguments
-  ctx = g_option_context_new("SLIDE - try opening a slide file");
+  GOptionContext *ctx =
+    g_option_context_new("SLIDE - try opening a slide file");
   g_option_context_add_main_entries(ctx, options, NULL);
-  if (!g_option_context_parse(ctx, &argc, &argv, &err)) {
-    fail("%s", err->message);
-    g_clear_error(&err);
+  if (!g_option_context_parse(ctx, &argc, &argv, &tmp_err)) {
+    fail("%s", tmp_err->message);
+    g_clear_error(&tmp_err);
     g_option_context_free(ctx);
     return 2;
   }
@@ -145,11 +147,12 @@ int main(int argc, char **argv) {
     fail("No slide specified");
     return 2;
   }
-  filename = argv[1];
+  const char *filename = argv[1];
 
   // Record preexisting file descriptors
-  fds = g_hash_table_new(g_direct_hash, g_direct_equal);
+  GHashTable *fds = g_hash_table_new(g_direct_hash, g_direct_equal);
   for (int i = 0; i < MAX_FDS; i++) {
+    struct stat st;
     if (!fstat(i, &st)) {
       g_hash_table_insert(fds, GINT_TO_POINTER(i), GINT_TO_POINTER(1));
     }
@@ -159,8 +162,22 @@ int main(int argc, char **argv) {
       (G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING),
       print_log, NULL);
 
-  can_open = openslide_can_open(filename);
-  osr = openslide_open(filename);
+  const char *vendor = openslide_detect_vendor(filename);
+  openslide_t *osr = openslide_open(filename);
+
+  // Check vendor if requested
+  if (vendor_check) {
+    const char *expected_vendor = vendor_check;
+    if (!strcmp(expected_vendor, "none")) {
+      expected_vendor = NULL;
+    }
+    if ((expected_vendor == NULL) != (vendor == NULL) ||
+        (vendor && strcmp(vendor, expected_vendor))) {
+      fail("Detected vendor %s, expected %s",
+           vendor ? vendor : "NULL",
+           expected_vendor ? expected_vendor : "NULL");
+    }
+  }
 
   // Check for open errors
   if (osr != NULL) {
@@ -170,14 +187,6 @@ int main(int argc, char **argv) {
     have_error = TRUE;
   }
 
-  // Make sure openslide_can_open() told the truth.
-  // This may produce false warnings if messages are logged through glib,
-  // but that already indicates a programming error.
-  if (can_open != !have_error) {
-    fail("openslide_can_open returned %s but openslide_open %s",
-         can_open ? "true" : "false", have_error ? "failed" : "succeeded");
-  }
-
   if (osr != NULL) {
     // Check properties and regions
     check_props(osr);
@@ -189,6 +198,7 @@ int main(int argc, char **argv) {
 
   // Check for file descriptor leaks
   for (int i = 0; i < MAX_FDS; i++) {
+    struct stat st;
     if (!fstat(i, &st) && !g_hash_table_lookup(fds, GINT_TO_POINTER(i))) {
       // leaked
       char *link_path = g_strdup_printf("/proc/%d/fd/%d", getpid(), i);
@@ -204,5 +214,41 @@ int main(int argc, char **argv) {
   }
   g_hash_table_destroy(fds);
 
+  // Do timing run.  The earlier openslide_open() doesn't count because
+  // it reads the slide data into the page cache.
+  if (time_check && !have_error) {
+    GTimer *timer = NULL;
+
+    // Average of TIME_ITERATIONS runs
+    for (int i = 0; i < TIME_ITERATIONS; i++) {
+      // Try open
+      if (timer) {
+        g_timer_continue(timer);
+      } else {
+        timer = g_timer_new();
+      }
+      osr = openslide_open(filename);
+      g_timer_stop(timer);
+
+      // Check for errors and clean up
+      if (osr != NULL) {
+        check_error(osr);
+        openslide_close(osr);
+      } else {
+        fail("openslide_open() returned NULL during timing loop");
+      }
+      if (have_error) {
+        break;
+      }
+    }
+
+    // Report results
+    if (!have_error) {
+      printf("%d ms\n",
+             (int) (1000 * g_timer_elapsed(timer, NULL) / TIME_ITERATIONS));
+    }
+    g_timer_destroy(timer);
+  }
+
   return have_error ? 1 : 0;
 }
